from base58 import b58decode, b58encode
from base64 import b64decode, b64encode

import re

regex_non_decimal = re.compile(r'''[^0-9,]''')
def decimal_csv_decode(line):
    line = regex_non_decimal.sub('', line)
    numbers = [int(x) for x in line.split(',') if x]
    return bytes(numbers)

regex_non_hex = re.compile(r'''[^0-9A-Fa-f,]''')
def hex_csv_decode(line):
    line = regex_non_hex.sub('', line)
    numbers = [int(x, base=16) for x in line.split(',') if x]
    return bytes(numbers)

def hex_decode(line):
    return bytes.fromhex(line)

from neo3.wallet import utils
from neo3.core import types
def address_to_hash160(address: str) -> bytes:
    '''Convert NEO address to script hash'''
    if not utils.is_valid_address:
        raise ValueError(f"not a valid address: {address}")
    return utils.address_to_script_hash(address).to_array()

def hash160_to_address(hash160: bytes) -> str:
    '''Convert NEO script hash to address'''
    h = types.UInt160(hash160)
    address = utils.script_hash_to_address(h)
    if not utils.is_valid_address:
        raise ValueError(f"not a valid address: {address}")
    return address

def unknown_hash160(hash160: bytes):
    '''Convert hash160 to possible NEO addresses'''
    result = {}
    try:
        result['address_straight'] = hash160_to_address(hash160)
    except Exception:
        pass
    try:
        result['address_reversed'] = hash160_to_address(reversed(hash160))
    except Exception:
        pass
    return result

def unknown_address(address: str):
    '''Convert address to possible representations of script hash'''
    h = address_to_hash160(address)
    r = bytes(reversed(h))
    return {
        'hash160_straight': h,
        'hash160_straight_hex': h.hex(),
        'hash160_straight_base58': b58encode(h),
        'hash160_reversed': r,
        'hash160_reverse_hex': r.hex(),
        'hash160_reverse_base58': b58encode(r),
    }

def xxd(b: bytes):
    '''Dump bytes to hex for human to read'''
    print(b.hex(' ', -2))

def is_valid_address_prefix(prefix: str) -> bool:
    '''Check if a given prefix is possible for NEO wallet address'''
    valid = 'NfrostC3BsMcVaNLLE1ihaMMuTit1PGBj9'
    if len(prefix) > len(valid):
        return False
    if len(prefix) == 0:
        return True
    if prefix[0] != valid[0]:
        return False
    try:
        raw = b58decode(prefix + valid[len(prefix):])
    except Exception as e:
        return False
    if raw[0] != 0x35:
        return False
    return True
